/**************************************************************************/
/*                                                                        */
/* Copyright (c) 2001, 2011 NoMachine, http://www.nomachine.com/.         */
/*                                                                        */
/* NXPROXY, NX protocol compression and NX extensions to this software    */
/* are copyright of NoMachine. Redistribution and use of the present      */
/* software is allowed according to terms specified in the file LICENSE   */
/* which comes in the source distribution.                                */
/*                                                                        */
/* Check http://www.nomachine.com/licensing.html for applicability.       */
/*                                                                        */
/* NX and NoMachine are trademarks of Medialogic S.p.A.                   */
/*                                                                        */
/* All rigths reserved.                                                   */
/*                                                                        */
/**************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <pthread.h>

#include <X11/Xutil.h>
#include <X11/Xmu/WinUtil.h>

#include "NX.h"

#define PANIC
#define WARNING
#undef  TEST
#undef  DEBUG

Window findNXagentWindow(Display *display);
void setWMClass(Display *display, Window window);
void *wmdisplay(void *args);

Window findNXagentWindow(Display *display) {
        //code taken from xlsclients from x11-utils

        Window dummy, *children = NULL, client = None;
        unsigned int i, nchildren = 0;

        char **cliargv = NULL;
        int cliargc;

#ifdef DEBUG
        fprintf(stderr, "called findNXagentWindow");
#endif
        // query the tree
        if (!XQueryTree(display, DefaultRootWindow(display), &dummy, &dummy, &children, &nchildren)) {
                return client;
        }

        // query all windows
        for (i = 0; i < nchildren; i++) {
                client = XmuClientWindow(display, children[i]);
                if (client != None) {
                         //search for the command name
                         if (!XGetCommand (display, client, &cliargv, &cliargc)) {
                              client = None;
                         } else {
                                // compare if it equals nxagent at normal positions
                                // ToDo maybee let the user decide where the nxagent on the server residents

                                if ( (cliargv[0] != NULL) &&
                                     (strlen(cliargv[0]) >= strlen("nxagent")) &&
                                     (strcmp(cliargv[0] + (strlen(cliargv[0]) - strlen("nxagent")), "nxagent") == 0) )
                                {
                                        break;
                                } else {
                                       client = None;
                                }
                                XFreeStringList (cliargv);
                         }
                }
        }
        // here we return the client window None or the window which comes from nxagent
        return client;
}

void setWMClass(Display *display, Window window)
{
         XClassHint *classHint;

         classHint = XAllocClassHint();

         classHint->res_name = "nxagent";
         classHint->res_class = "Nxagent";

         XSetClassHint(display, window, classHint);

         //clean up
         XFree(classHint);
}


void *wmdisplay(void *args)
{
        char *displayname = NULL;
        Display *display;
        Window window;
        unsigned int boolme=0;
        unsigned int count=0;

        display = XOpenDisplay (displayname);

        if(!display)
        {
                return NULL;
        }

        while (boolme < 1) {
                window = findNXagentWindow(display);

                if(window != None && count <= 30){
                        setWMClass(display,window);
                        boolme = 1;
                }
                sleep(1);
                count++;
        }

        XCloseDisplay (display);
	return NULL;
}

/*
 * Entry point when running nxproxy stand-alone.
 */

int main(int argc, const char **argv)
{

  pthread_t thread1;

  int tres;

  int result = -1;

  char *options = NULL;

  options = getenv("NX_DISPLAY");



  if ( (tres = pthread_create( &thread1, NULL, &wmdisplay, NULL)) != 0)
        {
#ifdef DEBUG
                                fprintf(stderr, "Thread creation failed: %d\n", tres);
#endif
  }



  if (NXTransParseCommandLine(argc, argv) < 0)
  {
    NXTransCleanup();
  }

  if (NXTransParseEnvironment(options, 0) < 0)
  {
    NXTransCleanup();
  }

  /*
   * This should not return...
   */

  #ifdef TEST
  fprintf(stderr, "Main: Yielding control to NX entry point.\n");
  #endif

  result = NXTransProxy(NX_FD_ANY, NX_MODE_ANY, NX_DISPLAY_ANY);

  /*
   * ...So these should not be called.
   */

  NXTransExit(result);

  return 0;
}
